﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;
using AIOUSBNetFragment;

namespace AIOUSBNetFragment
{
    public class AIOUSB
    {
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_StreamOpen(UInt32 DeviceIndex, UInt32 bIsRead);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_StreamClose(UInt32 DeviceIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 GetDevices();
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 ClearDevices();
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 AIOUSB_CloseDevice(UInt32 DeviceIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 ResolveDeviceIndex(UInt32 DeviceIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_ConfigureEx(UInt32 DeviceIndex, [In] Byte[] pOutMask, [In] Byte[] pData, [In] Byte[] pTristateMask);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_StreamSetClocks(UInt32 DeviceIndex, ref Double ReadClockHz, ref Double WriteClockHz);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_StreamFrame(UInt32 DeviceIndex, UInt32 FramePoints, byte[] pFrameData, ref UInt32 ByteTransferred);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_StreamFrame(UInt32 DeviceIndex, UInt32 FramePoints, UInt16[] pFrameData, ref UInt32 ByteTransferred);
    }
}



namespace ConsoleApplication1
{
    class Program
    {
        const UInt32 diOnly = 0xFFFFFFFD;

        private static void AIOUSB_ResetDLL()
        {
            for (uint i = 0; i < 32; i++)
                AIOUSB.AIOUSB_CloseDevice(i);
            AIOUSB.ClearDevices();
        }

        private static void buildBuffer(out UInt16[] buf)
        {
            uint i;
            buf = new UInt16[100000];
            for (i = 0; i < buf.Length; i++)
            {
                buf[i] = (UInt16)i;
            }
        }

        private static UInt32 DIO_StreamOutputFrame(UInt32 DeviceIndex, [In] UInt16[] wordBuf, [In] UInt32 startIndex, [In] UInt32 length, out UInt32 wordsTransfered)
        {
            UInt32 bytesTransferred = 0;
            UInt32 FramePoints = length;

            UInt32 Status = AIOUSB.DIO_StreamFrame(DeviceIndex, FramePoints, wordBuf, ref bytesTransferred);
            wordsTransfered = bytesTransferred / 2;
            return Status;
        }

        //This is the function that actually sends chunks of data(frames). This
        //  sample sends files, not realtime data, so this function just sends
        //  chunks until it's sent the whole file or there's an error.

        private static UInt32 DoTransferData(UInt32 deviceIndex, UInt32 bufferLength, UInt16[] wordBuf)
        {
            UInt32 status, wordsTransferred = 0, nextPt = 0, wordsLeft = bufferLength;

            DateTime startT = DateTime.Now, endT;
            do
            {

                status = DIO_StreamOutputFrame(deviceIndex, wordBuf, nextPt, wordsLeft, out wordsTransferred);

                if (status == 38)
                    status = 0; // If it's a partial, that's fine, we'll just loop around and send the rest.
                if (status != 0)
                    break;
                wordsLeft -= wordsTransferred;
                nextPt += wordsTransferred;
            } while (wordsLeft > 0);
            endT = DateTime.Now;
            Console.WriteLine("\r\nActual duration: {0}", endT - startT);
            return status;
        }

        static void Main(string[] args)
        {
            const int cOpenForWriting = 0;
            // const int cOpenForReading = 1;
            UInt32 status = 0;
            UInt32 devicesMask;
            UInt32 deviceIndex;
            Boolean bHasntRunOnce = true;

            while (bHasntRunOnce)
            {
                bHasntRunOnce = false; // once only. used to make error handling cleaner to read in the source

                Console.WriteLine("USB-DIO-16H family output sample, in C#.");
                Console.WriteLine("This sample is designed to operate a single USB-DIO16H family device.");
                Console.WriteLine("This sample generates a digital waveform file and transmits it at 20kHz.\r\n");

                AIOUSB_ResetDLL();

                devicesMask = AIOUSB.GetDevices();
                if (devicesMask == 0)
                {
                    Console.WriteLine("No cards found. Please address this issue and re-run the program");
                    status = 1;
                    break;
                }

                if ((deviceIndex = AIOUSB.ResolveDeviceIndex(diOnly)) == 0xFFFFFFFF) // confirm only one device, See note at bottom of code
                {
                    Console.WriteLine("\"diOnly\" didn't resolve to a deviceIndex; ");
                    status = 2;
                    break;
                }

                // no startup issues detected, assuming valid card type, here's the body of the code

                byte[] OutMask = { 0x07 }; //A, B, and C output; D is opposite C, so it's input.
                byte[] DIOData = { 0xFF, 0xFF, 0xFF, 0xFF };
                byte[] TristateMask = { 0x00 };

                status = AIOUSB.DIO_ConfigureEx(deviceIndex, OutMask, DIOData, TristateMask);
                if (status != 0) break;

                double readHz = 0.0, writeHz = 20000.0; 
                status = AIOUSB.DIO_StreamSetClocks(deviceIndex, ref readHz, ref writeHz);
                if (status != 0) break;
                Console.WriteLine("Actual output rate: {0:f4}", writeHz);

                status = AIOUSB.DIO_StreamOpen(deviceIndex, cOpenForWriting);
                if (status != 0) break;

                UInt16[] buf;
                buildBuffer(out buf);

                double duration = buf.Length / writeHz;

                Console.WriteLine("Writing {0} words of data at {1:f4}Hz, should take {2:f4} seconds", buf.Length, writeHz, duration);
                status = DoTransferData(deviceIndex, (UInt32)buf.Length, buf);
                if (status != 0) break;

                status = AIOUSB.DIO_StreamClose(deviceIndex);
                if (status != 0) break;
            }
            Console.WriteLine("Application finished with status == {0}\r\nPress any key to exit.", status);
            Console.ReadKey(true);

        }

    }
}
